#!/usr/bin/env python3

__author__ = "Jonathan Shimwell"


import os
import time
import unittest
from pathlib import Path

import openmc
from openmc_data_downloader import expand_materials_xml_to_isotopes
from openmc_data_downloader.utils import expand_elements_to_isotopes


class test_command_line_usage(unittest.TestCase):
    def test_single_isotope_download_endf_71_wmp(self):
        os.system("rm *.h5")
        os.system("openmc_data_downloader -l ENDFB-7.1-WMP -i H1")

        assert Path("ENDFB-7.1-WMP_H1.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_single_isotope_download_tendl_2019(self):
        os.system("rm *.h5")
        os.system("openmc_data_downloader -l TENDL-2019 -i H1")

        assert Path("TENDL-2019_H1.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_multiple_isotope_download(self):
        os.system("rm *.h5")
        os.system("openmc_data_downloader -l TENDL-2019 -i H1 He4")

        assert Path("TENDL-2019_H1.h5").is_file()
        assert Path("TENDL-2019_He4.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 2

    def test_correct_files_from_command_line_usage_2(self):
        os.system("rm *.h5")
        os.system("openmc_data_downloader -l TENDL-2019 -e F")

        assert Path("TENDL-2019_F19.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_correct_files_from_command_line_usage_3(self):

        os.system("rm *.h5")

        os.system("openmc_data_downloader -l ENDFB-7.1-NNDC -e Co Y")

        assert Path("ENDFB-7.1-NNDC_Co59.h5").is_file()
        assert Path("ENDFB-7.1-NNDC_Y89.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 2

    def test_correct_files_from_command_line_usage_4(self):

        os.system("rm *.h5 materials.xml")

        my_mat = openmc.Material()
        my_mat.add_element("Nb", 0.5)
        my_mat.add_nuclide("Cs133", 0.5)
        os.system("rm materials.xml")
        openmc.Materials([my_mat]).export_to_xml()

        os.system("openmc_data_downloader -l TENDL-2019 -m materials.xml")

        assert expand_materials_xml_to_isotopes(["materials.xml"]) == ["Nb93", "Cs133"]

        assert Path("TENDL-2019_Nb93.h5").is_file()
        assert Path("TENDL-2019_Cs133.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 2

    def test_correct_files_from_command_line_usage_5(self):
        """Tests downloading with FENDL"""
        os.system("rm *.h5 materials.xml")

        my_mat = openmc.Material()
        my_mat.add_element("Nb", 0.5)
        os.system("rm materials.xml")
        openmc.Materials([my_mat]).export_to_xml()

        os.system("openmc_data_downloader -l FENDL-3.1d -m materials.xml -p neutron")

        assert expand_materials_xml_to_isotopes(["materials.xml"]) == ["Nb93"]

        assert Path("FENDL-3.1d_Nb93.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_correct_files_from_command_line_usage_6(self):
        """Tests downloading with FENDL as prioity but without the element in
        FENDL"""

        os.system("rm *.h5")

        os.system("openmc_data_downloader -l FENDL-3.1d TENDL-2019 -e Pr")

        assert expand_elements_to_isotopes(["Pr"]) == ["Pr141"]

        assert Path("TENDL-2019_Pr141.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_photon_download_of_isotope_nndc(self):
        """Tests downloading with NNDC photon data and checks it exists"""

        os.system("rm *.h5")

        os.system("openmc_data_downloader -l ENDFB-7.1-NNDC -i He4 -p photon")

        assert Path("ENDFB-7.1-NNDC_He.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_photon_download_of_isotope_fendl(self):
        """Tests downloading with FENDL photon data and checks it exists"""

        os.system("rm *.h5")

        os.system("openmc_data_downloader -l FENDL-3.1d -i He4 -p photon")

        assert Path("FENDL-3.1d_He.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 1

    def test_neutron_and_photon_download_of_isotope_fendl(self):
        """Tests downloading with FENDL photon data and checks it exists"""

        os.system("rm *.h5")

        os.system("openmc_data_downloader -l FENDL-3.1d -i He4 -p photon neutron")

        assert Path("FENDL-3.1d_He.h5").is_file()
        assert Path("FENDL-3.1d_He4.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 2

    def test_sab_download_with_endf(self):
        """Tests downloading with FENDL photon data and checks it exists"""

        os.system("rm *.h5")

        os.system(
            "openmc_data_downloader -l ENDFB-7.1-NNDC TENDL-2019 -e Be O -s c_Be_in_BeO"
        )

        assert Path("ENDFB-7.1-NNDC_Be9.h5").is_file()
        assert Path("ENDFB-7.1-NNDC_O16.h5").is_file()
        assert Path("ENDFB-7.1-NNDC_O17.h5").is_file()
        assert Path("TENDL-2019_O18.h5").is_file()
        assert Path("ENDFB-7.1-NNDC_c_Be_in_BeO.h5").is_file()
        assert Path("materials.xml").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 5

    def test_download_single_file_with_overwrite_speed_up(self):
        """Checks that downloading with overwrite to False is quicker"""

        current_time = time.time()
        os.system(
            "openmc_data_downloader -l ENDFB-7.1-NNDC TENDL-2019 -e Be --overwrite"
        )
        time_after_download = time.time()
        time_to_download = time_after_download - current_time

        current_time = time.time()
        os.system(
            "openmc_data_downloader -l ENDFB-7.1-NNDC TENDL-2019 -e Be --no-overwrite",
        )
        time_after_download = time.time()
        time_to_not_download = time_after_download - current_time

        assert time_to_not_download < time_to_download



try:
    # this works for python 3.7 and lower
    from importlib.metadata import version, PackageNotFoundError
except (ModuleNotFoundError, ImportError):
    # this works for python 3.8 and higher
    from importlib_metadata import version, PackageNotFoundError
try:
    __version__ = version("openmc_data_downloader")
except PackageNotFoundError:
    from setuptools_scm import get_version

    __version__ = get_version(root="..", relative_to=__file__)

__all__ = ["__version__"]

from .cross_sections_directory import (
    NATURAL_ABUNDANCE,
    LIB_OPTIONS,
    PARTICLE_OPTIONS,
    SAB_OPTIONS,
    STABLE_ISOTOPE_OPTIONS,
    ALL_ISOTOPE_OPTIONS,
    xs_info,
    sab_info,
)

from .utils import (
    create_cross_sections_xml,
    just_in_time_library_generator,
    download_data_frame_of_isotopes,
    download_single_file,
    expand_elements_to_isotopes,
    expand_materials_xml_to_isotopes,
    expand_materials_xml_to_sab,
    identify_isotopes_to_download,
    expand_materials_to_isotopes,
    expand_materials_to_sabs,
    identify_sab_to_download,
)



import os
import warnings
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import List, Optional, Union
from urllib.parse import urlparse
from urllib.request import urlopen
from urllib.error import HTTPError
import pandas as pd
from retry import retry

from openmc_data_downloader import (
    ALL_ISOTOPE_OPTIONS,
    STABLE_ISOTOPE_OPTIONS,
    LIB_OPTIONS,
    NATURAL_ABUNDANCE,
    PARTICLE_OPTIONS,
    SAB_OPTIONS,
    sab_info,
    xs_info,
)

_BLOCK_SIZE = 16384


def set_environmental_variable(cross_section_xml_path: Union[Path, str]) -> None:

    if not isinstance(cross_section_xml_path, Path):
        cross_section_xml_path = Path(cross_section_xml_path)

    if cross_section_xml_path.is_file() is False:
        raise FileNotFoundError(
            "{} was not found, therefore not setting OPENMC_CROSS_SECTIONS "
            "environmental variable".format(cross_section_xml_path)
        )

    print("setting OPENMC_CROSS_SECTIONS", str(cross_section_xml_path))
    os.environ["OPENMC_CROSS_SECTIONS"] = str(cross_section_xml_path)


def expand_materials_to_isotopes(materials: list):

    if isinstance(materials, list):
        if len(materials) == 0:
            return []

    try:
        import openmc
    except ImportError:
        msg = (
            "import openmc failed. openmc python package could not be found "
            "and was not imported, the expand_materials_to_isotopes "
            "opperation can not be performed without openmc"
        )
        raise ImportError(msg)

    if isinstance(materials, openmc.Materials):
        iterable_of_materials = materials
    elif isinstance(materials, list):
        for material in materials:
            if not isinstance(material, openmc.Material):
                raise ValueError(
                    "When passing a list then each entry in the list must be "
                    "an openmc.Material. Not a",
                    type(material),
                )
        iterable_of_materials = materials
    elif isinstance(materials, openmc.Material):
        iterable_of_materials = [materials]
    else:
        raise ValueError(
            "materials must be of type openmc.Materials, openmc,Material or a "
            "list or openmc.Material. Not ",
            type(materials),
        )

    if len(iterable_of_materials) > 0:
        isotopes_from_materials = []
        for material in iterable_of_materials:
            for nuc in material.nuclides:
                isotopes_from_materials.append(nuc.name)

        return isotopes_from_materials

    return []


def expand_materials_to_sabs(materials: list):

    if isinstance(materials, list):
        if len(materials) == 0:
            return []

    try:
        import openmc
    except ImportError:
        msg = (
            "import openmc failed. openmc python package could not be found "
            "and was not imported, the expand_materials_to_sabs "
            "opperation can not be performed without openmc"
        )
        raise ImportError(msg)

    if isinstance(materials, openmc.Materials):
        iterable_of_materials = materials
    elif isinstance(materials, list):
        for material in materials:
            if not isinstance(material, openmc.Material):
                raise ValueError(
                    "When passing a list then each entry in the list must be "
                    "an openmc.Material. Not a",
                    type(material),
                )
        iterable_of_materials = materials
    elif isinstance(materials, openmc.Material):
        iterable_of_materials = [materials]
    else:
        raise ValueError(
            "materials must be of type openmc.Materials, openmc,Material or a "
            "list or openmc.Material. Not ",
            type(materials),
        )

    if len(iterable_of_materials) > 0:
        sabs_from_materials = []
        for material in iterable_of_materials:
            for sab_tuple in material._sab:
                sabs_from_materials.append(sab_tuple[0])

        return sabs_from_materials

    return []


def just_in_time_library_generator(
    libraries: List[str] = [],
    isotopes: List[str] = [],
    elements: List[str] = [],
    sab: List[str] = [],
    destination: Union[str, Path] = None,
    materials_xml: List[Union[str, Path]] = [],
    materials: list = [],  # also accepts a single openmc.Material
    particles: Optional[List[str]] = ["neutron", "photon"],
    set_OPENMC_CROSS_SECTIONS: bool = True,
    overwrite: bool = False,
) -> str:

    # expands elements, materials xml into list of isotopes

    isotopes_from_elements = expand_elements_to_isotopes(elements)
    isotopes = list(set(isotopes + isotopes_from_elements))

    isotopes_from_material_xml = expand_materials_xml_to_isotopes(materials_xml)
    isotopes = list(set(isotopes + isotopes_from_material_xml))

    isotopes_from_materials = expand_materials_to_isotopes(materials)
    isotopes = list(set(isotopes + isotopes_from_materials))

    # filters the large dataframe of all isotopes into just the ones you want
    dataframe_xs = identify_isotopes_to_download(
        libraries=libraries,
        particles=particles,
        isotopes=isotopes,
    )

    sab_from_material_xml = expand_materials_xml_to_sab(materials_xml)
    sab = list(set(sab + sab_from_material_xml))

    sabs_from_materials = expand_materials_to_sabs(materials)
    sab = list(set(sab + sabs_from_materials))

    dataframe_sab = identify_sab_to_download(
        libraries=libraries,
        sab=sab,
    )

    dataframe = pd.concat([dataframe_sab, dataframe_xs])

    download_data_frame_of_isotopes(
        dataframe=dataframe, destination=destination, overwrite=overwrite
    )

    cross_section_xml_path = create_cross_sections_xml(dataframe, destination)

    if set_OPENMC_CROSS_SECTIONS is True:
        # making the cross section xml requires openmc and returns None if
        # openmc is not found.
        if cross_section_xml_path is not None:
            set_environmental_variable(cross_section_xml_path)
    else:
        print(
            "Set your $OPENMC_CROSS_SECTIONS environmental variable to "
            f"{cross_section_xml_path} to use this custom library"
        )

    return cross_section_xml_path


def download_single_file(
    url: str,
    output_filename: Union[str, Path] = None,
    destination: Union[str, Path] = None,
    overwrite: bool = True,
) -> Path:
    """Download file from a URL

    Arguments:
        url: URL from which to download
        destination: Specifies a folder location to save the downloaded file

    Returns
        Name of file written locally
    """

    if output_filename is not None:
        if not isinstance(output_filename, Path):
            output_filename = Path(output_filename)

    if destination is not None:
        if not isinstance(destination, Path):
            destination = Path(destination)

    if output_filename is None:
        local_path = Path(Path(urlparse(url).path).name)
    else:
        local_path = output_filename

    if destination is not None:
        Path(destination).mkdir(parents=True, exist_ok=True)
        local_path = destination / local_path

    if overwrite is False and local_path.is_file():
        print("Skipping {}, already downloaded".format(local_path))
        return local_path

    local_path = download_url_in_chuncks(url, local_path)

    return local_path


@retry(HTTPError, tries=3)
def download_url_in_chuncks(url, local_path):
    with urlopen(url) as response:

        # Copy file to disk in chunks
        print("Downloading {}... ".format(local_path), end="")

        with open(local_path, "wb") as fh:
            while True:
                chunk = response.read(_BLOCK_SIZE)
                if not chunk:
                    break
                fh.write(chunk)
            print("")

    return local_path


def download_data_frame_of_isotopes(
    dataframe, destination: Union[str, Path], overwrite: bool = True
):

    local_files = []
    for index, row in dataframe.iterrows():
        local_file = download_single_file(
            url=row["url"],
            output_filename=row["local_file"],
            destination=destination,
            overwrite=overwrite,
        )
        local_files.append(local_file)

    return local_files


def create_cross_sections_xml(dataframe, destination: Union[str, Path]) -> str:

    try:
        import openmc
    except ImportError:
        msg = (
            "import openmc failed. openmc python package could not be found "
            "and was not imported, cross sections.xml can not be made"
            "without openmc"
        )
        warnings.warn(msg)
        return None

    library = openmc.data.DataLibrary()
    for index, row in dataframe.iterrows():
        if destination is None:
            library.register_file(Path(row["local_file"]))
        else:
            library.register_file(Path(destination) / Path(row["local_file"]))
    if destination is None:
        library.export_to_xml("cross_sections.xml")
        cross_sections_xml_path = "cross_sections.xml"
    else:
        if not isinstance(destination, Path):
            destination = Path(destination)
            destination.mkdir(parents=True, exist_ok=True)
        library.export_to_xml(destination / "cross_sections.xml")
        cross_sections_xml_path = str(destination / "cross_sections.xml")

    absolute_path = str(Path(cross_sections_xml_path).absolute())
    print(absolute_path, "written")

    return absolute_path


def identify_sab_to_download(
    libraries: List[str],
    sab: Optional[List[str]] = [],
):
    if sab == []:
        return pd.DataFrame()
    elif sab == "all" or sab == ["all"]:
        sab = SAB_OPTIONS

    priority_dict = {}

    if isinstance(libraries, str):
        libraries = [libraries]

    if isinstance(sab, str):
        sab = [sab]

    if libraries == []:
        raise ValueError(
            "At least one library must be selected, options are", LIB_OPTIONS
        )

    for counter, entry in enumerate(libraries):
        if entry not in LIB_OPTIONS:
            raise ValueError("The library must be one of the following", LIB_OPTIONS)

    for counter, entry in enumerate(sab):
        if entry not in SAB_OPTIONS:
            raise ValueError(
                "The sab argument must be one of the following",
                SAB_OPTIONS,
                " Not {entry}",
            )

        priority_dict[entry] = counter + 1

    print("Searching libraries with the following priority", priority_dict)

    # Tried to removed this dict to dataframe conversion out of the function
    # and into the initialization of the package but this resultied in
    # a SettingwithCopyWarning which can be fixed and understood here
    # https://www.dataquest.io/blog/settingwithcopywarning/
    sab_info_df = pd.DataFrame.from_dict(sab_info)

    is_library = sab_info_df["library"].isin(libraries)
    print("SaB found matching library requirements", is_library.values.sum())

    print("sab names", sab)
    is_sab = sab_info_df["name"].isin(sab)
    print("SaB found matching name requirements", is_sab.values.sum())

    sab_info_df = sab_info_df[(is_sab) & (is_library)]

    sab_info_df["priority"] = sab_info_df["library"].map(priority_dict)

    sab_info_df = sab_info_df.sort_values(by=["priority"])

    sab_info_df = sab_info_df.drop_duplicates(subset=["name"], keep="first")

    # end url is unique so this avoids downloading duplicates of the same file
    sab_info_df = sab_info_df.drop_duplicates(subset=["url"], keep="first")

    print("SaB found matching all requirements", len(sab_info_df))

    return sab_info_df


def identify_isotopes_to_download(
    libraries: List[str],
    particles: List[str] = [],
    isotopes: Optional[List[str]] = [],
):

    if isotopes == []:
        return pd.DataFrame()
    elif isotopes == "all" or isotopes == ["all"]:
        isotopes = ALL_ISOTOPE_OPTIONS
    elif isotopes == "stable" or isotopes == ["stable"]:
        isotopes = STABLE_ISOTOPE_OPTIONS

    print("isotopes", isotopes)

    priority_dict = {}

    if isinstance(libraries, str):
        libraries = [libraries]

    if isinstance(particles, str):
        particles = [particles]

    if libraries == []:
        raise ValueError(
            "At least one library must be selected, options are", LIB_OPTIONS
        )

    if particles == []:
        raise ValueError(
            "At least one particle type must be selected, options are", PARTICLE_OPTIONS
        )

    for counter, entry in enumerate(libraries):
        if entry not in LIB_OPTIONS:
            raise ValueError("The library must be one of the following", LIB_OPTIONS)

        priority_dict[entry] = counter + 1

    for counter, entry in enumerate(particles):
        if entry not in PARTICLE_OPTIONS:
            raise ValueError(
                "The library must be one of the following", PARTICLE_OPTIONS
            )

    print("Searching libraries with the following priority", priority_dict)

    # Tried to removed this dict to dataframe conversion out of the function
    # and into the initialization of the package but this resultied in
    # a SettingwithCopyWarning which can be fixed and understood here
    # https://www.dataquest.io/blog/settingwithcopywarning/
    xs_info_df = pd.DataFrame.from_dict(xs_info)

    is_library = xs_info_df["library"].isin(libraries)
    print("Isotopes found matching library requirements", is_library.values.sum())

    is_particle = xs_info_df["particle"].isin(particles)
    print("Isotopes found matching particle requirements", is_particle.values.sum())

    is_isotope = xs_info_df["isotope"].isin(isotopes)
    print("Isotopes found matching isotope requirements", is_isotope.values.sum())

    xs_info_df = xs_info_df[(is_isotope) & (is_library) & (is_particle)]

    xs_info_df["priority"] = xs_info_df["library"].map(priority_dict)

    xs_info_df = xs_info_df.sort_values(by=["priority"])

    xs_info_df = xs_info_df.drop_duplicates(
        subset=["isotope", "particle"], keep="first"
    )

    # end url is unique so this avoids downloading duplicates of the same file
    xs_info_df = xs_info_df.drop_duplicates(subset=["url"], keep="first")

    print("Isotopes found matching all requirements", len(xs_info_df))

    return xs_info_df


def expand_elements_to_isotopes(elements: Union[str, List[str]]):

    if elements == "stable" or elements == ["stable"]:
        return STABLE_ISOTOPE_OPTIONS

    if elements == "all" or elements == ["all"]:
        return ALL_ISOTOPE_OPTIONS

    if isinstance(elements, str):
        return NATURAL_ABUNDANCE[elements]

    isotopes = []
    for element in elements:
        isotopes = isotopes + NATURAL_ABUNDANCE[element]
    return isotopes


def expand_materials_xml_to_isotopes(
    materials_xml: Union[List[str], str] = "materials.xml"
):

    isotopes = []

    if isinstance(materials_xml, str):
        materials_xml = [materials_xml]

    if materials_xml == []:
        return []

    for material_xml in materials_xml:
        tree = ET.parse(material_xml)
        root = tree.getroot()

        for elem in root:
            for subelem in elem:
                if subelem.tag == "nuclide":
                    if "name" in subelem.attrib.keys():
                        isotopes.append(subelem.attrib["name"])
    return isotopes


def expand_materials_xml_to_sab(materials_xml: Union[List[str], str] = "materials.xml"):

    sabs = []

    if isinstance(materials_xml, str):
        materials_xml = [materials_xml]

    if materials_xml == []:
        return []

    for material_xml in materials_xml:
        tree = ET.parse(material_xml)
        root = tree.getroot()

        for elem in root:
            for subelem in elem:
                if subelem.tag == "sab":
                    if "name" in subelem.attrib.keys():
                        sabs.append(subelem.attrib["name"])
    return sabs



#!/usr/bin/env python3

__author__ = "Jonathan Shimwell"


import os
import unittest
from pathlib import Path

import openmc
from openmc_data_downloader import just_in_time_library_generator


class test_usage_with_openmc_python_api(unittest.TestCase):
    def test_materials_download(self):
        """openmc.Materials are a container for openmc.Material objects. This
        test checks that they are handeled correctly"""

        os.system("rm *.h5")
        os.system("rm my_custom_nuclear_data_with_materials/*.h5")

        # Define material
        my_mat_1 = openmc.Material()
        my_mat_1.add_nuclide("Pu239", 3.7047e-2)

        my_mat_2 = openmc.Material()
        my_mat_2.add_nuclide("Pu240", 1.7512e-3)
        my_mat_2.add_nuclide("As75", 1.3752e-3)
        mats = openmc.Materials([my_mat_1, my_mat_2])
        mats.export_to_xml()

        # Create a sphere of my_mat
        surf_1 = openmc.Sphere(r=1)
        surf_2 = openmc.Sphere(r=2, boundary_type="vacuum")
        cell_1 = openmc.Cell(fill=my_mat_1, region=-surf_1)
        cell_2 = openmc.Cell(fill=my_mat_1, region=surf_1 and -surf_2)
        openmc.Geometry([cell_1, cell_2]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 5
        settings.batches = 3
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.export_to_xml()

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        just_in_time_library_generator(
            destination="my_custom_nuclear_data_with_materials",
            libraries=["TENDL-2019"],
            materials=mats,
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path(
            "my_custom_nuclear_data_with_materials/TENDL-2019_Pu239.h5"
        ).is_file()
        assert Path(
            "my_custom_nuclear_data_with_materials/TENDL-2019_Pu240.h5"
        ).is_file()
        assert Path(
            "my_custom_nuclear_data_with_materials/TENDL-2019_As75.h5"
        ).is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.3.h5").is_file()

        assert (
            len(list(Path("my_custom_nuclear_data_with_materials").glob("*.h5"))) == 3
        )

    def test_simulation_with_destination(self):

        os.system("rm *.h5")
        os.system("rm my_custom_nuclear_data_dir/*.h5")

        # Define material
        my_mat = openmc.openmc.Material()
        my_mat.add_nuclide("Pu239", 3.7047e-2)
        my_mat.add_nuclide("Pu240", 1.7512e-3)
        my_mat.add_nuclide("As75", 1.3752e-3)
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.export_to_xml()

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        just_in_time_library_generator(
            destination="my_custom_nuclear_data_dir",
            libraries=["TENDL-2019"],
            materials=my_mat,
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("my_custom_nuclear_data_dir/TENDL-2019_Pu239.h5").is_file()
        assert Path("my_custom_nuclear_data_dir/TENDL-2019_Pu240.h5").is_file()
        assert Path("my_custom_nuclear_data_dir/TENDL-2019_As75.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()

        assert len(list(Path("my_custom_nuclear_data_dir").glob("*.h5"))) == 3

    def test_photon_simulation_with_single_mat(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.Material()
        my_mat.add_element("Fe", 1)
        openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        settings.run_mode = "fixed source"
        settings.photon_transport = True

        source = openmc.Source()
        source.space = openmc.stats.Point((0, 0, 0))
        source.angle = openmc.stats.Isotropic()
        # This is a Co60 source, see the task on sources to understand it
        source.energy = openmc.stats.Discrete([1.1732e6], [1.0])
        source.particle = "photon"

        settings.source = source
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["FENDL-3.1d"],
            materials=my_mat,
            # TODO find out why neutrons are needed here
            particles=["photon", "neutron"],
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("FENDL-3.1d_Fe.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()

        # normally 3 if just photons used
        assert len(list(Path(".").glob("*.h5"))) == 7

    def test_photon_neutron_simulation_with_single_mat(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.openmc.Material()
        my_mat.add_element("Fe", 1)
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        settings.photon_transport = True
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.run_mode = "fixed source"
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["FENDL-3.1d"],
            materials=my_mat,
            particles=["neutron", "photon"],
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("FENDL-3.1d_Fe54.h5").is_file()
        assert Path("FENDL-3.1d_Fe56.h5").is_file()
        assert Path("FENDL-3.1d_Fe57.h5").is_file()
        assert Path("FENDL-3.1d_Fe58.h5").is_file()
        assert Path("FENDL-3.1d_Fe.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 7  # summary and statepoint

    def test_simulation_with_single_mat(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.openmc.Material()
        my_mat.add_nuclide("Pu239", 3.7047e-2)
        my_mat.add_nuclide("Pu240", 1.7512e-3)
        my_mat.add_nuclide("As75", 1.3752e-3)
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["TENDL-2019"],
            materials=my_mat,
            particles="neutron",
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("TENDL-2019_Pu239.h5").is_file()
        assert Path("TENDL-2019_Pu240.h5").is_file()
        assert Path("TENDL-2019_As75.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 5  # summary and statepoint

    def test_simulation_with_sab(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.Material()
        my_mat.add_element("Be", 0.5)
        my_mat.add_s_alpha_beta("c_Be_in_BeO")
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.run_mode = "fixed source"
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["ENDFB-7.1-NNDC"],
            materials=my_mat,
            particles="neutron",
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("ENDFB-7.1-NNDC_Be9.h5").is_file()
        assert Path("ENDFB-7.1-NNDC_c_Be_in_BeO.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 4  # summary and statepoint

    def test_wmp_simulation_with_single_mat(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.openmc.Material()
        my_mat.add_nuclide("P31", 1)
        my_mat.temperature = 300  # main_cell used instead
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        main_cell.temperature = 300
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        settings.photon_transport = False
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.run_mode = "fixed source"
        settings.temperature = {"multipole": True}
        settings.export_to_xml()

        just_in_time_library_generator(
            # 'ENDFB-7.1-NNDC' is needed as WMP simulations appear to require
            # non WMP cross sections as well when simulating
            # https://openmc.discourse.group/t/minimal-wmp-simulation-with-minimal-cross-section-xml/1100/7
            libraries=["ENDFB-7.1-NNDC", "ENDFB-7.1-WMP"],
            materials=my_mat,
            particles="neutron",
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()
        os.system("ls")

        # WMP is currently not downloaded due to this issue
        # https://github.com/openmc-data-storage/openmc_data_downloader/issues/13
        # assert Path('ENDFB-7.1-WMP_P31.h5').is_file()
        assert Path("ENDFB-7.1-NNDC_P31.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()
        assert len(list(Path(".").glob("*.h5"))) == 3

    def test_simulation_with_single_mat_list(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat = openmc.openmc.Material()
        my_mat.add_nuclide("Pu239", 3.7047e-2)
        my_mat.add_nuclide("Pu240", 1.7512e-3)
        my_mat.add_nuclide("As75", 1.3752e-3)
        openmc.openmc.Materials([my_mat]).export_to_xml()

        # Create a sphere of my_mat
        surf = openmc.Sphere(r=6.3849, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat, region=-surf)
        openmc.Geometry([main_cell]).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["TENDL-2019"],
            materials=[my_mat],
            particles="neutron",
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("TENDL-2019_Pu239.h5").is_file()
        assert Path("TENDL-2019_Pu240.h5").is_file()
        assert Path("TENDL-2019_As75.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()

        assert len(list(Path(".").glob("*.h5"))) == 5  # summary and statepoint

    def test_simulation_with_multi_mat_list(self):

        os.system("rm *.h5")

        # this clears the enviromental varible just to be sure that current
        # system settings are not being used
        try:
            del os.environ["OPENMC_CROSS_SECTIONS"]
        except KeyError:
            # key not found on system
            pass

        # Define material
        my_mat1 = openmc.openmc.Material()
        my_mat1.add_nuclide("Pu239", 3.7047e-2)
        my_mat1.add_nuclide("Pu240", 1.7512e-3)

        my_mat2 = openmc.openmc.Material()
        my_mat2.add_element("As", 1.3752e-3)
        openmc.openmc.Materials([my_mat1, my_mat2]).export_to_xml()

        # Create a sphere of my_mat
        surf1 = openmc.Sphere(r=1)
        surf2 = openmc.Sphere(r=6, boundary_type="vacuum")
        main_cell = openmc.Cell(fill=my_mat1, region=-surf1)
        outer_cell = openmc.Cell(fill=my_mat2, region=+surf1 & -surf2)
        universe = openmc.Universe(cells=[main_cell, outer_cell])
        openmc.Geometry(universe).export_to_xml()

        # Define settings for the simulation
        settings = openmc.Settings()
        settings.particles = 10
        settings.batches = 2
        settings.inactive = 0
        center = (0.0, 0.0, 0.0)
        settings.source = openmc.Source(space=openmc.stats.Point(center))
        settings.export_to_xml()

        just_in_time_library_generator(
            libraries=["TENDL-2019"],
            materials=[my_mat1, my_mat2],
            set_OPENMC_CROSS_SECTIONS=True,
        )

        os.system("echo $OPENMC_CROSS_SECTIONS")
        openmc.run()

        assert Path("TENDL-2019_Pu239.h5").is_file()
        assert Path("TENDL-2019_Pu240.h5").is_file()
        assert Path("TENDL-2019_As75.h5").is_file()

        assert Path("summary.h5").is_file()
        assert Path("statepoint.2.h5").is_file()

        assert len(list(Path(".").glob("*.h5"))) == 5  # summary and statepoint



#!/usr/bin/env python3

__author__ = "Jonathan Shimwell"


import os
import unittest
import time
import pytest

import openmc
import pandas as pd
from openmc_data_downloader import (
    expand_materials_to_isotopes,
    expand_materials_xml_to_isotopes,
    identify_isotopes_to_download,
    identify_sab_to_download,
    expand_elements_to_isotopes,
    expand_materials_xml_to_sab,
    expand_materials_to_sabs,
    download_single_file,
)


def test_expansion_of_elements_with_stable_keyword():
    all_stable_isotopes = expand_elements_to_isotopes("stable")
    assert len(all_stable_isotopes) == 290


def test_expansion_of_elements_with_all_keyword():
    all_stable_isotopes = expand_elements_to_isotopes("all")
    assert len(all_stable_isotopes) == 1233


def test_expansion_of_elements_with_single_element():
    stable_isotopes = expand_elements_to_isotopes("Be")
    assert stable_isotopes == ["Be9"]


def test_expansion_of_elements_with_multiple_element():
    stable_isotopes = expand_elements_to_isotopes(["Be", "Li"])
    assert stable_isotopes == ["Be9", "Li6", "Li7"]


def test_identify_sab_to_download_with_all_keyword():

    filtered_df = identify_sab_to_download(libraries=["ENDFB-7.1-NNDC"], sab=["all"])

    assert len(filtered_df.values) == 20

    filtered_df2 = identify_sab_to_download(libraries=["ENDFB-7.1-NNDC"], sab="all")

    assert len(filtered_df2.values) == 20


def test_identify_sab_to_download_finds_two():

    filtered_df = identify_sab_to_download(
        libraries=["ENDFB-7.1-NNDC"], sab=["c_Be_in_BeO", "c_H_in_H2O"]
    )

    assert len(filtered_df.values) == 2


def test_identify_isotopes_to_download_finds_tendl_neutron():

    filtered_df = identify_isotopes_to_download(
        libraries=["TENDL-2019"], isotopes=["Be9"], particles=["neutron"]
    )
    answer_df = pd.DataFrame.from_dict(
        {
            "isotope": ["Be9"],
            "particle": ["neutron"],
            "library": ["TENDL-2019"],
            "remote_file": ["Be9.h5"],
            "url": [
                "https://github.com/openmc-data-storage/TENDL-2019/raw/main/h5_files/Be9.h5"
            ],
            "element": ["Be"],
            "local_file": ["TENDL-2019_Be9.h5"],
            "priority": [1],
        }
    )
    print(filtered_df)
    assert len(filtered_df.values) == 1
    assert list(filtered_df.keys()) == list(answer_df.keys())
    assert filtered_df.values[0].tolist() == answer_df.values[0].tolist()

    # can't get pandas dataframe comparison to work so resorted to the lists above
    # assert_frame_equal(answer_df, filtered_df)


def test_identify_isotopes_to_download_finds_fendl_photon():

    filtered_df = identify_isotopes_to_download(
        libraries=["FENDL-3.1d"], isotopes=["Be9"], particles=["photon"]
    )
    answer_df = pd.DataFrame.from_dict(
        {
            "isotope": ["Be9"],
            "particle": ["photon"],
            "library": ["FENDL-3.1d"],
            "remote_file": ["Be.h5"],
            "url": [
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/photon/Be.h5"
            ],
            "element": ["Be"],
            "local_file": ["FENDL-3.1d_Be.h5"],
            "priority": [1],
        }
    )

    assert len(filtered_df.values) == 1
    assert list(filtered_df.keys()) == list(answer_df.keys())
    assert filtered_df.values[0].tolist() == answer_df.values[0].tolist()


def test_identify_isotopes_to_download_finds_fendl_photon_neutron():

    filtered_df = identify_isotopes_to_download(
        libraries=["FENDL-3.1d"], isotopes=["Be9"], particles=["photon", "neutron"]
    )
    answer_df = pd.DataFrame.from_dict(
        {
            "isotope": ["Be9", "Be9"],
            "particle": ["neutron", "photon"],
            "library": ["FENDL-3.1d", "FENDL-3.1d"],
            "remote_file": ["Be9.h5", "Be.h5"],
            "url": [
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/neutron/Be9.h5",
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/photon/Be.h5",
            ],
            "element": ["Be", "Be"],
            "local_file": ["FENDL-3.1d_Be9.h5", "FENDL-3.1d_Be.h5"],
            "priority": [1, 1],
        }
    )

    assert len(filtered_df.values) == 2
    assert list(filtered_df.keys()) == list(answer_df.keys())
    assert filtered_df.values[0].tolist() == answer_df.values[0].tolist()
    assert filtered_df.values[1].tolist() == answer_df.values[1].tolist()


def test_identify_isotopes_to_download_finds_fendl_photon_neutron_multi_isotopes():

    filtered_df = identify_isotopes_to_download(
        libraries=["FENDL-3.1d"],
        isotopes=["Fe56", "Fe57"],
        particles=["photon", "neutron"],
    )
    answer_df = pd.DataFrame.from_dict(
        {
            "isotope": ["Fe56", "Fe57", "Fe56"],
            "particle": ["neutron", "neutron", "photon"],
            "library": ["FENDL-3.1d", "FENDL-3.1d", "FENDL-3.1d"],
            "remote_file": ["Fe56.h5", "Fe57.h5", "Fe.h5"],
            "url": [
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/neutron/Fe56.h5",
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/neutron/Fe57.h5",
                "https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/photon/Fe.h5",
            ],
            "element": ["Fe", "Fe", "Fe"],
            "local_file": [
                "FENDL-3.1d_Fe56.h5",
                "FENDL-3.1d_Fe57.h5",
                "FENDL-3.1d_Fe.h5",
            ],
            "priority": [1, 1, 1],
        }
    )

    assert len(filtered_df.values) == 3
    assert list(filtered_df.keys()) == list(answer_df.keys())
    assert filtered_df.values[0].tolist() == answer_df.values[0].tolist()
    assert filtered_df.values[1].tolist() == answer_df.values[1].tolist()
    assert filtered_df.values[2].tolist() == answer_df.values[2].tolist()


def test_identify_isotopes_to_download_all():

    filtered_df = identify_isotopes_to_download(
        libraries=["FENDL-3.1d"], isotopes=["all"], particles=["photon", "neutron"]
    )

    assert len(filtered_df.values) == 239

    filtered_df = identify_isotopes_to_download(
        libraries=["FENDL-3.1d"], isotopes="all", particles=["photon", "neutron"]
    )

    assert len(filtered_df.values) == 239


def test_expand_materials_from_object_list_with_single_mat():

    my_mat = openmc.Material()
    my_mat.add_nuclide("Pu239", 3.7047e-2)
    my_mat.add_nuclide("Pu240", 1.7512e-3)
    my_mat.add_nuclide("Pu241", 1.1674e-4)

    assert expand_materials_to_isotopes([my_mat]) == ["Pu239", "Pu240", "Pu241"]


def test_expand_materials_from_object_with_single_mat():

    my_mat = openmc.Material()
    my_mat.add_nuclide("Pu239", 3.7047e-2)
    my_mat.add_nuclide("Pu240", 1.7512e-3)
    my_mat.add_nuclide("Pu241", 1.1674e-4)

    assert expand_materials_to_isotopes(my_mat) == ["Pu239", "Pu240", "Pu241"]


def test_expand_materials_from_object_list_with_multiple_mat():

    my_mat1 = openmc.Material()
    my_mat1.add_nuclide("Li6", 0.5)
    my_mat1.add_nuclide("Li7", 0.25)

    my_mat2 = openmc.Material()
    my_mat2.add_nuclide("Al27", 0.25)

    assert expand_materials_to_isotopes([my_mat1, my_mat2]) == [
        "Li6",
        "Li7",
        "Al27",
    ]


def test_expand_materials_from_object_list_with_openmc_materials():

    my_mat1 = openmc.Material()
    my_mat1.add_nuclide("Li6", 0.5)
    my_mat1.add_nuclide("Li7", 0.25)

    my_mat2 = openmc.Material()
    my_mat2.add_nuclide("Al27", 0.25)

    mats = openmc.Materials([my_mat1, my_mat2])

    assert expand_materials_to_isotopes(mats) == ["Li6", "Li7", "Al27"]


def test_expand_material_xmls_with_list_input():

    my_mat = openmc.Material()
    my_mat.add_nuclide("Be9", 0.5)
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    assert expand_materials_xml_to_isotopes(["materials.xml"]) == ["Be9"]


def test_expand_material_xmls_with_str_input():

    my_mat = openmc.Material()
    my_mat.add_element("Al", 0.5)
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    assert expand_materials_xml_to_isotopes("materials.xml") == ["Al27"]


def test_expand_material_xmls_with_two_isotopes():

    my_mat = openmc.Material()
    my_mat.add_element("Li", 0.5)
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    assert "Li6" in expand_materials_xml_to_isotopes("materials.xml")
    assert "Li7" in expand_materials_xml_to_isotopes("materials.xml")


def test_expand_material_xmls_with_sab():

    my_mat = openmc.Material()
    my_mat.add_element("Be", 0.5)
    my_mat.add_s_alpha_beta("c_Be_in_BeO")
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    # sab should not be in this list as this is just isotopes
    assert expand_materials_xml_to_isotopes("materials.xml") == ["Be9"]


def test_expand_material_xmls_for_sabs_with_sab():

    my_mat = openmc.Material()
    my_mat.add_element("Be", 0.5)
    my_mat.add_s_alpha_beta("c_Be_in_BeO")
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    assert expand_materials_xml_to_sab("materials.xml") == ["c_Be_in_BeO"]


def test_expand_material_xmls_for_sabs_with_two_sab():

    my_mat = openmc.Material()
    my_mat.add_element("Be", 0.5)
    my_mat.add_s_alpha_beta("c_Be_in_BeO")
    my_mat.add_s_alpha_beta("c_H_in_H2O")
    os.system("rm materials.xml")
    openmc.Materials([my_mat]).export_to_xml()

    assert expand_materials_xml_to_sab("materials.xml") == [
        "c_Be_in_BeO",
        "c_H_in_H2O",
    ]


def test_expand_material_for_sabs_with_two_sab():

    my_mat = openmc.Material()
    my_mat.add_element("Be", 0.5)
    my_mat.add_s_alpha_beta("c_Be_in_BeO")
    my_mat.add_s_alpha_beta("c_H_in_H2O")

    assert expand_materials_to_sabs(my_mat) == ["c_Be_in_BeO", "c_H_in_H2O"]


def test_expand_material_for_sabs_with_sab():

    my_mat = openmc.Material()
    my_mat.add_element("Be", 0.5)
    my_mat.add_s_alpha_beta("c_H_in_H2O")

    assert expand_materials_to_sabs(my_mat) == ["c_H_in_H2O"]


def test_incorrect_material_enpty():
    with pytest.raises(ValueError):
        expand_materials_to_sabs("my_mat")


def test_incorrect_sab_name():
    with pytest.raises(ValueError):
        identify_sab_to_download(libraries=["ENDFB-7.1-NNDC"], sab=["incorrect name"])


def test_incorrect_libraries():
    with pytest.raises(ValueError):
        identify_sab_to_download(libraries=[], sab=["c_Fe56"])


def test_incorrect_library_name_for_sab_identifying():
    with pytest.raises(ValueError):
        identify_sab_to_download(libraries=["incorrect name"], sab=["c_Fe56"])


def test_library_values_single_entry_list():

    isotopes_df = identify_isotopes_to_download(
        libraries="TENDL-2019", isotopes=["Li6", "Al27"], particles=["neutron"]
    )

    assert len(isotopes_df) == 2


def test_emplty_isotopes():
    empty_df = identify_isotopes_to_download(
        libraries=["TENDL-2019"], isotopes=[], particles=["neutron"]
    )
    assert len(empty_df) == 0
    assert isinstance(empty_df, type(pd.DataFrame()))


def test_incorrect_library_values_empty():
    with pytest.raises(ValueError):
        identify_isotopes_to_download(
            libraries=[], isotopes="Li6", particles=["neutron"]
        )


def test_incorrect_library_values_wrong():
    with pytest.raises(ValueError):
        identify_isotopes_to_download(
            libraries=["coucou"], isotopes="Li6", particles=["neutron"]
        )


def test_incorrect_expand_materials_to_isotopes_with_incorrect_args():
    """Checks than an error is raised when incorrect values of materials
    are passed"""

    with pytest.raises(ValueError):
        expand_materials_to_isotopes(1)

    with pytest.raises(ValueError):
        expand_materials_to_isotopes([1, 2, 3])


def test_download_single_file_with_overwrite_speed_up():
    """Checks that downloading with overwrite to False is quicker"""

    current_time = time.time()
    download_single_file(
        url="https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/photon/Fe.h5",
        output_filename="Fe56_new_download.h5",
        overwrite=True,
    )
    time_after_download = time.time()
    time_to_download = time_after_download - current_time

    current_time = time.time()
    download_single_file(
        url="https://github.com/openmc-data-storage/FENDL-3.1d/raw/main/h5_files/photon/Fe.h5",
        output_filename="Fe56_new_download.h5",
        overwrite=False,
    )
    time_after_download = time.time()
    time_to_not_download = time_after_download - current_time

    assert time_to_not_download < time_to_download
